
#include <sys/signal.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <time.h>
#include <fcntl.h>
#include <limits.h>
#include <errno.h>
#include <termios.h>
#include <asm/byteorder.h>

#include "defines.h"

/*
 * Key definitions
 */

#define VC_KEY_NONE   0
#define VC_KEY_F1     1
#define VC_KEY_F2     2
#define VC_KEY_F3     3
#define VC_KEY_F4     4
#define VC_KEY_F5     5
#define VC_KEY_F6     6
#define VC_KEY_F7     7
#define VC_KEY_F8     8
#define VC_KEY_F9     9
#define VC_KEY_F10   10
#define VC_KEY_UP    11
#define VC_KEY_DOWN  12
#define VC_KEY_RIGHT 13
#define VC_KEY_LEFT  14
#define VC_KEY_HOME  15
#define VC_KEY_END   16
#define VC_KEY_PGUP  17
#define VC_KEY_PGDN  18

/*
 * Colors
 */

/* Main foreground 16 colors */
#define VC_BLACK         0
#define VC_BLUE          1
#define VC_GREEN         2
#define VC_CYAN          3
#define VC_RED           4
#define VC_MAGENTA       5
#define VC_BROWN         6
#define VC_LIGHTGRAY     7
#define VC_DARKGRAY      8
#define VC_LIGHTBLUE     9
#define VC_LIGHTGREEN   10
#define VC_LIGHTCYAN    11 
#define VC_LIGHTRED     12
#define VC_LIGHTMAGENTA 13
#define VC_YELLOW       14
#define VC_WHITE        15

/* Background color attributes */
#define VC_BG_BLUE          0x10
#define VC_BG_GREEN         0x20
#define VC_BG_CYAN          0x30
#define VC_BG_RED           0x40
#define VC_BG_MAGENTA       0x50
#define VC_BG_BROWN         0x60
#define VC_BG_LIGHTGRAY     0x70

#define VC_COLOR_ATTENTION  ( VC_WHITE     | VC_BG_RED       )
#define VC_COLOR_BACKCYAN   ( VC_BLACK     | VC_BG_CYAN      )
#define VC_COLOR_BACKBLUE   ( VC_LIGHTCYAN | VC_BG_BLUE      )
#define VC_COLOR_BACKGRAY   ( VC_BLACK     | VC_BG_LIGHTGRAY )

/* Set color off */
#define TERM_COLOR_OFF      0x2000

/*
 * Line drawing
 */

#define RAW_LINE_TOPLEFT       'l'
#define RAW_LINE_TOPRIGHT      'k'
#define RAW_LINE_BOTTOMLEFT    'm'
#define RAW_LINE_BOTTOMRIGHT   'j'
#define RAW_LINE_VERT          'x'
#define RAW_LINE_HORIZ         'q'
#define RAW_LINE_NOTHING       ' '
#define RAW_CROSS              'n'  /* a cross for box drawing */
#define RAW_LTEE               't'  /* tee pointing left       */
#define RAW_RTEE               'u'  /* tee pointing right      */
#define RAW_BTEE               'v'  /* tee pointing up         */
#define RAW_TTEE               'w'  /* tee pointing down       */
#define RAW_LARROW             ','  /* arrow pointing left     */
#define RAW_RARROW             '+'  /* arrow pointing right    */
#define RAW_DARROW             '.'  /* arrow pointing down     */
#define RAW_UARROW             '-'  /* arrow pointing up       */

#define FLAG_EDIT           2
#define FLAG_BELL           3
#define FLAG_NOHEX          5
#define FLAG_READONLY       6

#define EDIT_BITOP_XOR  0
#define EDIT_BITOP_OR   1
#define EDIT_BITOP_AND  2
#define EDIT_BITOP_NEG  3
#define EDIT_BITOP_ADD  4
#define EDIT_BITOP_SUB  5

#define EDIT_ASK_SAVE   0
#define EDIT_ASK_QUIT   1

/*
 * Global variables
 */

/* note: extern variables are explained where defined, just use grep. */

extern int fd;
extern struct stat stbuf;

extern int flag_bell;
extern int flag_edit;
extern int flag_nohex;
extern int flag_readonly;

extern int bytes_to_read;
extern int bytes_per_row;
extern int bytes_per_row_hex;

extern int ex, ey;

extern int bread;

/*
 * the windows size structure
 * (filled by init_screen_size in init.c)
 */
struct wscrn {
       unsigned short lines;
       unsigned short cols;
       unsigned short xp;
       unsigned short yp; 
} scrn;

/* The double linked list file structure */
struct ll_file {
        char *filename;
        long filepos;
        struct ll_file *next;
        struct ll_file *prev;
};

extern struct ll_file *ll_file_head;
extern struct ll_file *ll_file_current;
extern struct ll_file *ll_file_tail;
extern int ll_file_qty;

extern struct ll_file *ll_file_browse_curr;
extern struct ll_file *ll_file_browse_top;
extern int ll_file_browse_pos;

/* The linked list for editing changes */
struct ll_edit {
       char byte;
       long filepos;
       struct ll_edit *next;
};

extern struct ll_edit *ll_edit_head;

extern char hex_trans[];
extern char *obuffer;
extern char *restore_buffer;
extern int stop_restore;
extern int current_color;

extern long fpos;

extern int edit_mask;

extern char srch[];

/* edit.c */
void edit( void);
void set_char_colorxy( char byte, int attr);

/* file.c */
#define file_getpos() lseek( fd, (long) 0, SEEK_CUR);
int file_go_next( void);
int file_go_prev( void);
void file_open( char *filename);
void file_cannot_open( void);
void file_next_line( void);
void file_prev_line( void);
void file_next_byte( void);
void file_prev_byte( void);
void file_go_home( void);
void file_go_end( void);
void file_next_block( void);
void file_prev_block( void);

/* help.c */
void help( void);
void show_gnu_license( void);
void ascii_table( void);

/* hex.c */
void hex_mainloop( int view_or_edit);
void hex_showfile( void);
void hex_fill_hex_buffer( char *buf, char *hex_buffer, int size);
void hex_fill_offset_buffer( char *offset_buffer, int pos);
void hex_fill_ascii_buffer( char *buf, char *ascii_buffer, int size);

/* init.c */
void init( void);
void init_buffers( void);

/* keyboard.c */
void key_wait_for_any( void);
void key_scanf( char *str, int leng);
int key_process( void);

/* linkedlist.c */
void ll_file_init( void);
struct ll_file *ll_file_alloc( char *filename);
void ll_file_add( char *filename);
void ll_file_insert( char *filename);
void ll_file_free( void);
struct ll_file *ll_file_find( char *filename);
void ll_file_delete( void);
void ll_file_browse( void);
void ll_file_browse_show( void);
int ll_file_browse_next( int nitems);
int ll_file_browse_back( int nitems);
struct ll_edit *ll_edit_alloc( char byte);
void ll_edit_add( char byte, long filepos, struct ll_edit *tmp);
void ll_edit_free( void);

/* misc.c */
void do_exit( int status);
void draw_filename( char *filename);
void draw_main_screen( void);
void draw_fkeys( void);
void draw_offset_n_perc( long offset, long filesize);
void set_flag( int flag);
void *allocate_mem( int size);
char *trim_spaces( char *s);
void set_hexmode_variables( void);
int check_readonly( void);
char *replicate( char c, int size);

/* term.c */
void term_init( void);
void term_noraw( void);
void term_raw( void);
void term_beep( void);
void term_color( int color);
void term_unhide_cursor( void);
void term_hide_cursor( void);
void term_goto( int x, int y);
void term_enable_graf_map( void);
void term_disable_graf_map( void);

/* search.c */
void search( void);
void search_again( void);
void filegoto( void);
void search_set_color( void);

/* rawlib.c */
void raw_box( int x, int y, int xlen, int ylen, int attr);
void raw_buffer_puts( int x, int y, int size, char *str, int attr);
void raw_puts( int x, int y, int size, char *str);
void raw_putc( int x, int y, char byte);
void raw_save_screen( void);
void raw_load_screen( void);
void raw_dialog_box( char *titlestr, char *valuestr, int size, int tattr, int vattr);
void raw_fill_screen( int attr);
void raw_show_alert_box( char *str, int addy);
void raw_show_screentext( char *text, char *title);
void raw_do_title( char *title);
void raw_put_bottom_line( char *text);
void raw_flush( void);
int raw_ask( char *msg, char *q1, char *q2, char *q3, int posit);
void raw_show_message( char *msg, int addy);
void raw_fill_obuffer( void);

/* signals.c */
void signals_init( void);

